//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//
// rects
//
// by Christopher Rasmussen
//
// CISC 440/640, 2014
//
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <GL/glut.h>

//----------------------------------------------------------------------------

float alpha_degs = 20;
float beta_degs = 40;

//----------------------------------------------------------------------------

// draws a filled 1 x 1 square with lower-left corner at origin

void drawSquare()
{
  glColor3f (1.0, 1.0, 1.0);
  glBegin(GL_POLYGON);
  glVertex2f (0.0, 0.0);
  glVertex2f (0.0, 1.0);
  glVertex2f (1.0, 1.0);
  glVertex2f (1.0, 0.0);
  glEnd();
}

//----------------------------------------------------------------------------

void draw_axes()
{
  
  glColor3f (1.0, 0.0, 0.0);

  glPushMatrix();

    glBegin(GL_LINES);
  
    glVertex2f(-2, 0);
    glVertex2f(13, 0);
  
    glVertex2f(0, -2);
    glVertex2f(0, 13);
    
    glEnd();

  glPopMatrix();
}

//----------------------------------------------------------------------------

// first angle = alpha, second angle = beta

void display(void)
{
  glClear (GL_COLOR_BUFFER_BIT);

  draw_axes();

  // this enclosing push/pop pair is needed or else the translate/rotate below
  // will accumulate if display() is called multiple times

  glPushMatrix();

    // first rectangle

    glPushMatrix();
      glScalef(3, 2, 1);
      drawSquare();
    glPopMatrix();

    // this rotates and moves BOTH second and third rects

    glTranslatef(3, 2, 0);
    glRotatef(alpha_degs, 0, 0, 1);
 
    glPushMatrix();

      // second rectangle

      glPushMatrix();
        glScalef(5, -2, 1);          // "minus" means it is reflected across x axis so rotation is around upper-left corner
        drawSquare();
      glPopMatrix();
   
      // third rectangle

      glTranslatef(5, 0, 0);         // move to end of second rectangle
      glRotatef(beta_degs, 0, 0, 1);    
      glScalef(4, -2, 1);            // "minus" means it is reflected across x axis so rotation is around upper-left corner
      drawSquare();

    glPopMatrix();
 
  glPopMatrix();

  glFlush ();
}

//----------------------------------------------------------------------------

void keyboard(unsigned char key, int x, int y)
{
  float delta_theta = 2.0;

  if (key == 'q')
    exit(1);

  if (key == 'd' && alpha_degs >= delta_theta)
    alpha_degs -= delta_theta;
  else if (key == 'a' && alpha_degs <= 90 - delta_theta) 
    alpha_degs += delta_theta;
  else if (key == 's' && beta_degs >= delta_theta) 
    beta_degs -= delta_theta;
  else if (key == 'w' && beta_degs <= 90 - delta_theta) 
    beta_degs += delta_theta;
  
  glutPostRedisplay();
}

//----------------------------------------------------------------------------

void init (void) 
{
   glClearColor (0.0, 0.0, 0.0, 0.0);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D(-1.0, 14.0, -1.0, 14.0);
}

//----------------------------------------------------------------------------

int main(int argc, char** argv)
{
   glutInit(&argc, argv);
   glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB);
   glutInitWindowSize (250, 250); 
   glutInitWindowPosition (100, 100);
   glutCreateWindow ("rects");
   init ();

   glutDisplayFunc(display); 
   glutKeyboardFunc(keyboard);

   printf("a/d: move first joint\n");
   printf("w/s: move second joint\n");
   printf("q to quit\n");

   glutMainLoop();
   return 0;   
}

//----------------------------------------------------------------------------
